import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Scanner;

public class LessonReader {

    /*
    This class reads a text file and creates a lesson from it. The text file needs to be a certain structure. The structure
    is clearly defined in the ReadMe. This class has been designed so that the quiz for a lesson can have a variable amount
    of questions and these questions can have a variable amount of answers related to them.

    Please Note: If the Text File is not written how it's specified in the ReadMe this class may fail. However, it does
    have fall back values so that the user will see that the lesson has not been inputted correctly.
     */

    /**
     * Converts a text file to an ArrayList<String> line by line, each line being a new entry
     * @param fileName In the form "fileName.txt"
     * @return Text File as ArrayList<String>
     */
    public ArrayList<String> createArrayListFromTextFile(String fileName) {

        Scanner scanner = null;
        ArrayList<String> stringArrayList = new ArrayList<>();

        try {
            scanner = new Scanner(new BufferedReader(new FileReader(fileName)));

            while (scanner.hasNext()){
                stringArrayList.add(scanner.nextLine());
            }

        } catch (IOException e) {
            System.err.println(e.getMessage());
        } finally {
            scanner.close();
        }
        return stringArrayList;
    };

    /**
     * Creates a lesson from the Text File that you give it, Text file must be formatted a specific
     * way, this can be found in the ReadMe.
     * @param fileName in the form "fileName.txt"
     * @return Lesson object built from text file
     */
    public Lesson createLessonFromTextFile(String fileName) {

        /* Create the variables needed to build a lesson and give them fallback values
        if the text file provided is empty.
        */
        String lessonTitle = "This lesson doesn't have a title";
        String lessonText = "This lesson doesn't have any text";
        String questionString = "";
        int correctAnswer = 1;

        // Scans in the text file and gives the function the ArrayList to work with
        ArrayList<String> textArray = createArrayListFromTextFile(fileName);

        /* The answers input for a Question object and the questionsArray input for a Lesson object
        both require the data in an array, however to make the program more flexible and resilient and allow different
        amounts of answers and questions these are originally initialised as ArrayLists and then converted
         */
        ArrayList<String> answersArrayList = new ArrayList<>();
        ArrayList<Question> questionsArrayList = new ArrayList<>();


        // Array is looped through to get the title and Lesson text
        for(int i = 0; i < textArray.size(); i++) {
            if(textArray.get(i).equals("## Title")) {
                lessonTitle = textArray.get(i + 1);
            }
            if(textArray.get(i).equals("## Lesson Text")) {
                lessonText = textArray.get(i + 1);
            }
            if(textArray.get(i).equals("## Questions")) {

                /* Once Array gets to the Questions a second loop is created so that each question can
                be created separately, this allows for the text file to have different amounts of questions
                */
                for(int j = i; j < textArray.size(); j++) {
                    if (textArray.get(j).equals("# Question")) {
                        questionString = textArray.get(j + 1);
                    }
                    if (textArray.get(j).equals("# Answers")) {
                        /*
                        A while loop goes through the answers adding them to the answersArrayList,
                        This allows for each question to have a different amount of answers

                        Design Decision: Though having multiple loops inside each other is bad practice
                        it is unlikely that a text file will get to the size where this has any negative performance
                        impact and allowing different questions to have different amounts of answers
                        adds to the programs' user experience and flexibility.
                         */
                        boolean endOfAnswers = false;
                        while(!endOfAnswers) {
                            j += 1;
                            if(textArray.get(j).equals("# Correct Answer")) {
                                correctAnswer = Integer.parseInt(textArray.get(j + 1));
                                endOfAnswers = true;
                            } else {
                                answersArrayList.add(textArray.get(j));
                            }
                        }
                        // Converts an Array List into an array
                        String[] answerArray = new String[answersArrayList.size()];
                        answerArray = answersArrayList.toArray(answerArray);

                        // Creates the Question and adds it to question Array
                        Question question = new Question(questionString, answerArray, correctAnswer);
                        questionsArrayList.add(question);

                        // Removes the answers to the previous question so the array List can be reused
                        answersArrayList.clear();
                    }
                }
            }
        }
        // Converts the ArrayList of Questions to an Array which can be used to create the lesson
        Question[] questionArray = new Question[questionsArrayList.size()];
        questionArray = questionsArrayList.toArray(questionArray);

        // Creates the Lesson and returns it so that it can be used in the program
        Lesson lesson = new Lesson(lessonTitle,lessonText, questionArray);
        return lesson;
    }
}

