import java.util.ArrayList;

public class UsersStorage {

    /*
    This class stores a list of the Custom object User to record each user's progress while the program is running.
    It has been put into its own class due to the need to have some custom methods to interact with the storage.
     */

    private ArrayList<User> users;

    public UsersStorage() {
        this.users = new ArrayList<>();
    }

    // GETTER METHODS
    public ArrayList<User> getUsers() {
        return users;
    }
    // END GETTER METHODS

    // CUSTOM METHODS

    /**
     * Adds a user to the Storage ArrayList
     * @param user Custom User object
     */
    public void add(User user) {
        users.add(user);
    }


    /**
     * Check if the username entered already exists in Storage ArrayList
     * @param username Username you want to check
     * @return Returns -1 if the user doesn't exist, or returns the users place in the Storage ArrayList if they do
     */
    public int checkIfUserExists(String username) {
        for(int i = 0; i < users.size(); i++){
            String userNameInStorage = users.get(i).getUsername();
            if(userNameInStorage.equals(username)) {
                return i;
            }
        }
     return -1;
    }

    /**
     * Returns a string to display to the user based on if the user has completed just the lesson
     * or the lesson and the quiz and prints the score if applicable
     * @return Custom String based on input
     */
    public String printOutUserCompletionDataForLesson(Lesson lesson) {
        String lessonTitle = lesson.getLessonTitle();
        if(lesson.isQuizCompleted() == false) {
            return "Has completed the lesson for " + lessonTitle + " but not the quiz.";
        } else {
            return  "Has completed the lesson and quiz for " + lessonTitle + ", with a score of " + lesson.getQuizScore() + "/" + lesson.getQuestionArrayLength() + ".";
        }
    }

    /*
    Design Decision: The method below is not strictly necessary but has been included so that it is easy to check if
    the storage is working. Removing it could simplify the Program, Class and related ScannerExtension.
     */
    /**
     * Prints out all the data in the Storage Array List
     */
    public void printAllUsersData() {
        for(int i = 0; i < users.size(); i++) {
            /* Get a user from the array and loop over their record printing out each lesson,
             quiz and quiz score stored for them.
             Repeat this for every user stored in Storage ArrayList
            */
            User currentUser = users.get(i);
            System.out.println(currentUser.getUsername() + ": ");
            for (int j = 0; j < currentUser.getUserRecord().size(); j++) {
                System.out.println(printOutUserCompletionDataForLesson(currentUser.getUserRecord().get(j)));
            }
            /*
            Design Decision: Though having two nested arrays results in O(i * j) it is unlikely in this program that
            the arrays would get big enough for this to cause a performance issue. However, if the program was to grow
            it may be worth looking at more efficient ways to do this.
             */
        }
        // Fallback string, to be shown if no user data has been entered but user asks to see all user data.
        if(users.size() == 0) {
            System.out.println("No user data is stored at this time.");
        }
    }
}
