public class Lesson {

    /*
    This class creates a lesson object along with the ability to print the lesson and print and complete the quiz
    associated with it.
     */

    /* Design Decision:
       Part [1] - Lesson Title, Lesson and QuestionArray are marked as final as once initialised they will not change
       Part [2] - Lesson + Quiz Completed and QuizScore record what parts of a lesson each user has completed
       Makes logical sense to store completion data with the lesson, however as code gets larger it may be useful to
       split part [2] into a separate class as you don't need to store all of part [1], only the lessonTitle
       in the user record. This could save on memory.
       It is currently written this way to simplify the reading of the program, reducing the time needed to go through it.
     */
    final private String lessonTitle;
    final private String lessonText;
    final private Question[] questionArray;
    private boolean lessonCompleted;
    private boolean quizCompleted;
    private int quizScore;

    /**
     * When a lesson is initialised the user hasn't seen or completed the lesson or quiz so these are set to false and zero
     * @param lessonTitle
     * @param lessonText
     * @param questionArray
     */
    public Lesson(String lessonTitle, String lessonText, Question[] questionArray) {
        this.lessonTitle = lessonTitle;
        this.lessonText = lessonText;
        this.questionArray = questionArray;
        this.lessonCompleted = false;
        this.quizCompleted = false;
        this.quizScore = 0;
    }

    /* GETTER METHODS
    3 Standard getter methods, 1 custom getter method
    */
    public String getLessonTitle() {
        return lessonTitle;
    }

    public int getQuizScore() {
        return quizScore;
    }

    public boolean isQuizCompleted() {
        return quizCompleted;
    }

    // Rather than have a getter and then call .length on it, it has been combined here for readability.
    public int getQuestionArrayLength() {
        return questionArray.length;
    }

    // END GETTER METHODS

    /* SETTER METHODS
    3 fields are marked as final so can't be set
    */
    public void setQuizCompleted(boolean quizCompleted) {
        this.quizCompleted = quizCompleted;
    }

    /**
     * Checks if current quiz score is greater than saved score and replaces it.
     * This ensures that users best quiz score is saved.
     * @param quizScore
     */
    public void setQuizScore(int quizScore) {
        if(quizScore > this.quizScore) {
            this.quizScore = quizScore;
        }
    }

    // END SETTER METHODS

    // CUSTOM METHODS

    /**
     * Prints out the lesson title followed by the lesson text with empty lines in between for styling
     * <p>
     * Sets lessonCompleted to true as the lesson has now been printed out for user to read
     */
    public void printLesson() {
        // Prints out lesson title and text and sets the lesson completed to true
        System.out.println("You have chosen the lesson: " + lessonTitle);
        System.out.println(" ");
        System.out.println(lessonText);
        System.out.println(" ");
        lessonCompleted = true;
    }

    /**
     * Iterates over questionArray printing out each question in turn, recording user score,
     * and giving user immediate feedback on the quiz
     * @return total correct user answers
     */
    public int doQuiz() {
        // Initialise the users score and questions answered so they can see how they are doing after every question
        int correctAnswers = 0;
        int questionsAnswered = 0;

        // Styling so it is easy to see where the quiz starts
        System.out.println("  + ------------------------------------------------ +");
        System.out.println(" +  |                     Quiz                     |  +");
        System.out.println("  + ------------------------------------------------ +");
        System.out.println(" ");

        for(int i = 0; i < questionArray.length; i++) {
            // Checks that user got the question correct and adds 1 to score if they do
            if(questionArray[i].askQuestion(i + 1)) {
                correctAnswers += 1;
            }
            questionsAnswered += 1;

            // Checks to see if it is the last questions and outputs a different string depending on if it is or not.
            if (i != questionArray.length - 1) {
                System.out.print("Your current score is " + correctAnswers + "/" + questionsAnswered + ".");
            } else {
                System.out.println("Your final score is " + correctAnswers + "/" + questionsAnswered + ".");
                System.out.println("Congratulations on completing the lesson and the quiz.");
            }
            // Creates gap before next question
            System.out.println(" ");
            System.out.println(" ");
        }
        // Returns the users total score for the quiz so this can be recorded
        return correctAnswers;
    }

    // END CUSTOM METHODS

}
