public class Question {

    /*
    This class creates a Question which can be presented to the user.
    It has been designed so that the question can have as many answers
    as you wish to present to the user.
     */

    private String question;
    private String[] answers;
    private int correctAnswer;

    /**
     * Basic all arguments Constructor
     * @param question Just include the question String, no question number or punctuation before
     * @param answers Just include the answer String, no answer number or punctuation before
     * @param correctAnswer Add an int relating to the position of the correct answer based on array starting at 1, not 0.
     *                      (E.G. if you have 4 answers and the correct one is number 3 then use 3.)
     */
    public Question(String question, String[] answers, int correctAnswer) {
        this.question = question;
        this.answers = answers;
        this.correctAnswer = correctAnswer;
    }

    /**
     * Prints the Question for the user, the user can then pick an answer.
     * <p>
     * Returns a Boolean, true = user answered correctly, false = user answered incorrectly.
     * @param questionNumber used to format the question string displayed to the user.
     */
    public boolean askQuestion(int questionNumber) {
        /* Design decision. This was a choice between using a bool to represent correct or incorrect
        or using 1 to represent a correct answer and 0 for an incorrect answer. I decided on a boolean so that it
        reduces the use of "Magic Numbers" in the code, and it makes the code more resilient changes.
         */

        // Prints question with corresponding number
        System.out.println("Question " + questionNumber + ": " + question);
        System.out.println(" ");

        // Empty Strings declared which will be used to build the userInstruction string
        String userInstruction = "";
        String numberToChooseLesson = "";

        // Prints out the answers with their corresponding number and adds it to the userInstruction string
        for(int i = 0; i < answers.length; i++) {
            System.out.println((i + 1) + ". " + answers[i]);
            /* Check if it is the last answer, if so the string that is added to the userInstruction
             changes to represent this.
             */
            if(i != answers.length - 1) {
                numberToChooseLesson = (i + 1) + ", ";
            } else {
                numberToChooseLesson = (i + 1) + "):";
            }
            userInstruction = userInstruction + numberToChooseLesson;
        }

        // Prints out the userInstruction string with all the valid choices a user can make
        System.out.println(" ");
        System.out.println("Choose your answer by pressing the corresponding number (" + userInstruction);

        // Takes in the user input and checks if they got it right and then returns a boolean based on that
        int userChoice = ScannerExtension.scanForUserIntChoice(answers.length);

        if(userChoice == correctAnswer) {
            System.out.println("Correct!");
            return true;
        } else {
            System.out.println("Incorrect");
        }
        return false;
    };
}
